function out = FitLee_T_capilary(varargin)
% RPA fit for polydisperse polymer blends.
% Input parameters: 
%   Contrast = (b1/v1 - b2/v2)^2
%               bx : the unit scattering length density of sample x
%               vx : the segment volume of each components.
%               bx/vx : the scattering length density
%   Rgx : The radius of gyration (A) of x
%   chi : Chi parameter
%   vx : the segmental volume (cm^3/mol) of x
%   Nwx : weight averaged degree of polymerization of x
%         if monodisperse, set Nwx = 0;
%   Nnx : number averaged degree of polymerization of x
%   phix : volume fraction of x
%
% Byeongdu Lee
% Ref:
% B. Lee at al. Polymer, 51, 24, 12, 57995806.
% B. Lee et al. J. Appl. Cryst.;2009; 42; 161-168.
%
% See also RPAblend.m 
FitLee_helpstr = {'RPA fit for polydisperse polymer blends. ' ,...
'Input parameters:  ',...
'  Contrast = (b1/v1 - b2/v2)^2 ',...
'              bx : the unit scattering length density of sample x ',...
'              vx : the segment volume of each components. ',...
'              bx/vx : the scattering length density ',...
'      Here, it is just a constant.',...
'  RgNx : The radius of gyration (A) of x ',...
'       RgN = N*l/sqrt(6)',...
'       Relation between RgN and Rgz is found in ref 2',...
'  chix1000 : Chi parameter x 1000 ',...
'  vx : the segmental volume (cm^3/mol) of x ',...
'  Nwx : weight averaged degree of polymerization of x ',...
'        if monodisperse, set Nwx = 0 ',...
'  Nnx : number averaged degree of polymerization of x ',...
'  phix : volume fraction of x ',...
'Byeongdu Lee (blee@anl.gov)',...
'Ref: ',...
'    1. B. Lee at al. Polymer, 2010, 51, 24, 57995806. ',...
'    2. B. Lee et al. J. Appl. Cryst., 2009, 42, 161-168. '};

if numel(varargin) > 1
    p = varargin{1};
    q = varargin{2};
    isini = 0;
elseif numel(varargin) == 1
    p = varargin{1};
    isini = 1;
    if ischar(p)
        out = FitLee_helpstr;
        return
    end
elseif numel(varargin) == 0
    p = 1;
    isini = 1;
end

%% initialize fit parameter bestP
if isini
    Nf = p;
    bestP = [];
    bestP.u = 1.9;
    bestP.r = 0.3;
    bestP.y0 = 0.0;
    bestP.r3 = 0.3;
    bestP.r4 = 0.3;
    bestP.r5 = 0.3;
    bestP.r6 = 0.3;
    assignin('base', 'bestP', bestP);
    out = bestP;
    return
end

%% fitting codes .......................
if iscell(q);
    if numel(q) > 1
        error('FitLee_T_capilary.m is for fitting a set of data, for now')
    end
    q = q{1};
end
q = q(:);


out = exp(-p.u*2*sqrt(p.r^2 - (q-p.y0).^2));

if isnan(out)
    out = ones(size(out));
end