function [Sq, r,pr] = debyepr(q, x, y, z, resolution, atm, Fq)
%function [Sq, r,pr] = debyepr(q, x, y, z, resolution, atm, Fq)
% Calculate the structure factor (Sq) = iq + 1 using the Debye formulae
% It will calculate Sq using all atoms without any resolution reduction.
% p(r) function will be first calculated.
% The exact calculation see debyepr0.m that does not require resolution
% reduction.. but about at leat 2 times slower.
% q : Scattering vector, Nx1 array.
% x, y, and z: Mx1 array of positions of atoms.
% atm : Mx1 array dictating the type of atoms for each x.
%       its value corresponds to the column number of Fq and the maximum
%       value should be L.
% Fq : Scattering factor of atoms, NxL array.
%
% Byeongdu Lee.
% 
% Note that if there are atoms(particles) at the same locations, the
% calculated result may not be correct. Make sure no more than 1 atom per a
% location. Make sure like below:
% pv = [x,y,z];
% [dist, ff, i, j, ang] = distNzero(pv);
% k = find(dist<0.01);
% pv(j(k), :) = [];


if nargin< 5
    resolution = [];
    atm = [];
    Fq = [];
end
if nargin < 6
    atm = [];
    Fq = [];
end

if isempty(resolution)
    resolution = 0.1/max(q);
end

coord = [x(:),y(:),z(:)];

if isempty(atm)
    option = 1; % input: only the positions
else
    L = unique(atm);
    if numel(L) == 1
        option = 2; % single type of atoms;
    else
        option = 3; % multiple type of atoms;
    end
end

switch option
    case {1, 2}
        [r, pr] = distNzero1(coord, resolution);
        %dr = (r(2)-r(1));
        Sq = debyef0(q, r, pr)+numel(x);
        if option == 2
            Sq = Sq.*abs(Fq(:, L)).^2;
        end
    case 3
        fprintf('Recommended to use debyem.m for multi-type particles\n')
        fprintf('debyem.m will be way faster for multi-type particles\n')
        Sq = zeros(size(q));
        for i=1:numel(q)
            qv = q(i);
            F = Fq(i, atm);
            [r, pr] = distNzero1(coord, resolution, F);
            Sq(i) = debyef0(qv, r, pr)+sum(abs(F).^2);
        end
    otherwise
        cprintf('Input error')
end    