function polyhedron=polyhedra(varargin)
%one of five regular polyhedra
%in a format consistent with hieracrhical 
%modeler scaled to fit inside a unit sphere
% polyhedra('tetrahedron', edgelength, heightofTH)
% polyhedra('tetrahedron', [edgelength, heightofTH])
% polyhedra('tetrahedron')
% polyhedra('icosahedron')
% polyhedra('box', [xmin, xmax, ymin, ymax, zmin, zmax])
if numel(varargin) > 0
    ptype = varargin{1};
end

if numel(varargin) > 1
    edgelength = varargin{2};
else
    edgelength = [];
end

if numel(varargin) == 3
    tetheight = varargin{3};
else
    tetheight = [];
end
if isempty(tetheight) && numel(edgelength)>1
    tetheight = edgelength(2:end);
    edgelength = edgelength(1);
end

if (strcmp(ptype,'box'))
    edgelength = [edgelength, tetheight];
    xmin = edgelength(1);
    xmax = edgelength(2);
    ymin = edgelength(3);
    ymax = edgelength(4);
    zmin = edgelength(5);
    zmax = edgelength(6);
    dbox = [xmax-xmin, ymax-ymin, zmax-zmin];
    polyhedron.vertices=[...
         0 0 0; 1 0 0; 1 1 0; 0 1 0; ...
         0 0 1; 1 0 1; 1 1 1; 0 1 1;] ;
    polyhedron.faces=[ 1 2 6 5; 2 3 7 6; 3 4 8 7; 4 1 5 8; ...
         1 2 3 4; 5 6 7 8; ] ;
    polyhedron.vertices = polyhedron.vertices.*...
        repmat(dbox, size(polyhedron.vertices, 1), 1)+...
        [xmin, ymin, zmin];
elseif (strcmp(ptype,'tetrahedron'))
   polyhedron.vertices=[...
         1  -1/sqrt(3)  0
      -1 -1/sqrt(3) 0
      0 2/sqrt(3)  0
      0  0 sqrt(2/3)*2
   ];
   if ~isempty(edgelength)
       polyhedron.vertices = polyhedron.vertices/2*edgelength;
   end
   if ~isempty(tetheight)
       polyhedron.vertices(4,3) = tetheight;
   end
   polyhedron.faces=1+[...
         0 1 2
      0 1 3
      0 2 3
      1 2 3
   ];

elseif (strcmp(ptype, 'cylinder'))
    %% Computation of mesh coordinates
    N = 32;

% radius of cylinder
    r = edgelength;
    H = tetheight;
% extreme points of cylinder
    p1 = [0,0,-1]*H/2;
    p2 = [0,0,1]*H/2;

% compute orientation angle of cylinder
    [theta phi rho] = cart2sph2d(p2 - p1);
    dphi = linspace(0, 2*pi, N+1);
    dphi(end) = [];

% generate a cylinder oriented upwards
    x = repmat(cos(dphi) * r, [2 1]);
    y = repmat(sin(dphi) * r, [2 1]);
    z = repmat([0 ; rho], [1 length(dphi)]);

% transform points 
    trans   = localToGlobal3d(p1, theta, phi, 0);
    pts     = transformPoint3d([x(:) y(:) z(:)], trans);

% reshape transformed points
    x2 = reshape(pts(:,1), size(x));
    y2 = reshape(pts(:,2), size(x));
    z2 = reshape(pts(:,3), size(x));
    polyhedron.vertices = [[x2(1,:)';x2(2,:)'],...
        [y2(1,:)';y2(2,:)'],...
        [z2(1,:)';z2(2,:)'],];
    polyhedron.faces{1} = [1:N, 1];
    polyhedron.faces{2} = [(N+1):(2*N), N+1];
    for i=1:(N-1)
        polyhedron.faces{2+i} = [i, i+1, i+N+1, i+N];
    end
    polyhedron.faces{N+2} = [N, 1, N+1, 2*N];
elseif (strcmp(ptype,'pyramid'))
   polyhedron.vertices=[...
         1  1  0
      1 -1 0
      -1 -1  0
      -1 1 0
      0 0 sqrt(2)
   ];
   if ~isempty(edgelength)
       polyhedron.vertices = polyhedron.vertices/2*edgelength;
   end
   if ~isempty(tetheight)
       polyhedron.vertices(5,3) = tetheight;
   end
   polyhedron.faces={...
         [1 2 3 4],
      [1 2 5], 
      [2 3 5],
      [3 4 5],
      [4 1 5]
   };

elseif (strcmp(ptype,'directangulartetrahedron'))
    L=2; H = L/sqrt(2); 
    N = varargin{2};
    edgelength = L;
    tetheight = H;
    if numel(varargin) == 4
        edgelength = varargin{3};
        tetheight = varargin{4};
    end
   polyhedron.vertices=[...
         L/2, -L/(2*tan(pi/N)), 0
         -L/2, -L/(2*tan(pi/N)), 0
         0, 0, 0
         0, 0, H];
   if ~isempty(edgelength)
       polyhedron.vertices = polyhedron.vertices/2*edgelength;
   end
   if ~isempty(tetheight)
       polyhedron.vertices(4,3) = tetheight;
   end
   polyhedron.faces=[...
      1 2 3;
      1 2 4;
      2 3 4;
      3 1 4;
   ];

elseif (strcmp(ptype,'octant'))
    R = varargin{2}; 
    L = varargin{3};
    H = varargin{4};
    yv = sqrt(R^2-(L/2)^2);
    polyhedron.vertices=[...
         L/2, -yv, 0
         -L/2, -yv, 0
         0, 0, 0
         0, 0, H];

     polyhedron.faces=[...
      1 2 3;
      1 2 4;
      2 3 4;
      3 1 4;
   ];
   
elseif (strcmp(ptype,'cube'))
   polyhedron.vertices=0.5774*[...
         0 0 0; 1 0 0; 1 1 0; 0 1 0; ...
         0 0 1; 1 0 1; 1 1 1; 0 1 1;] ;
   
   polyhedron.vertices=polyhedron.vertices*2-0.5774;
   if ~isempty(edgelength)
       polyhedron.vertices = polyhedron.vertices/(2*0.5774)*edgelength;
   end
   
   polyhedron.faces=[ 1 2 6 5; 2 3 7 6; 3 4 8 7; 4 1 5 8; ...
         1 2 3 4; 5 6 7 8; ] ;

elseif (strcmp(ptype,'cuboid'))
    if isempty(tetheight)
        tetheight = edgelength;
    end
   polyhedron.vertices=[...
         0 0 0; edgelength 0 0; edgelength edgelength 0; 0 edgelength 0; ...
         0 0 tetheight; edgelength 0 tetheight; edgelength edgelength tetheight; 0 edgelength tetheight;] ;
   
   %polyhedron.vertices=polyhedron.vertices*2-0.5774;
   %if ~isempty(edgelength)
   %    polyhedron.vertices = polyhedron.vertices/(2*0.5774)*edgelength;
   %end
   polyhedron.vertices(:,1) = polyhedron.vertices(:,1) - edgelength/2;
   polyhedron.vertices(:,2) = polyhedron.vertices(:,2) - edgelength/2;
   polyhedron.vertices(:,3) = polyhedron.vertices(:,3) - tetheight/2;
   polyhedron.faces=[ 1 2 6 5; 2 3 7 6; 3 4 8 7; 4 1 5 8; ...
         1 2 3 4; 5 6 7 8; ] ;
   
elseif (strcmp(ptype,'octahedron'))
   polyhedron.vertices=[...
         0.408248  0.707107 -0.577350
      0.408248 -0.707107 -0.577350
      0.816496  0.000000  0.577350
      -0.408248  0.707107  0.577350
      -0.408248 -0.707107  0.577350
      -0.816496  0.000000 -0.577350
   ];
tanth = 0.577350/0.816496;
th = pi/2 - atan(tanth);
M = [cos(th), 0, -sin(th);
    0 1 0;
    sin(th), 0, cos(th)];
%M = [1,0,0;0,1,0;0,0,1];
v = (M*polyhedron.vertices')';
   if ~isempty(edgelength)
       polyhedron.vertices = v/(0.57735*sqrt(6))*edgelength;
   else
       polyhedron.vertices = v;
   end

%polyhedron.vertices = ;
   polyhedron.faces=1+[...
         3 2 0
      3 0 5
      5 0 1
      0 2 1
      3 5 4
      5 1 4
      4 1 2
      4 2 3
   ];

elseif (strcmp(ptype,'THH'))
    if numel(varargin)==3
        a = varargin{2};
        b = varargin{3};
        
        b = b-a;
    end

    polyhedron.vertices=[
        0, 0, (a+b)/2+3*a/14;
        a/2, -a/2, (a+b)/2;
        a/2, a/2, (a+b)/2;
        -a/2, a/2, (a+b)/2;
        -a/2, -a/2, (a+b)/2;
        a/2+3*a/14, 0, b/2;
        0, a/2+3*a/14, b/2;
        -a/2-3*a/14, 0, b/2;
        0, -a/2-3*a/14, b/2;
        a/2+3*a/14, 0, -b/2;
        0, a/2+3*a/14, -b/2;
        -a/2-3*a/14, 0, -b/2;
        0, -a/2-3*a/14, -b/2;
        a/2, -a/2, -(a+b)/2;
        a/2, a/2, -(a+b)/2;
        -a/2, a/2, -(a+b)/2;
        -a/2, -a/2, -(a+b)/2;
        0, 0, -(a+b)/2-3*a/14;];
    polyhedron.faces={
        [1,2,3];
        [1,3,4];
        [1,4,5];
        [1,5,2];
        [2,3,6];
        [3,4,7];
        [4,5,8];
        [5,2,9];
        [2,6,10,14];
        [6,3,15,10];
        [3,7,11,15];
        [7,4,16,11];
        [4,8,12,16];
        [8,5,17,12];
        [5,9,13,17];
        [9,2,14,13];
        [14,10,15];
        [15,11,16];
        [16,12,17];
        [17,14,13];
        [14,15,18];
        [15,16,18];
        [16,17,18];
        [17,14,18]};
        
        
   
elseif (strcmp(ptype,'dodecahedron'))
    phi = (1+sqrt(5))/2;


    vert = [1 1 1
        -1 1 1
        1 -1 1
        1 1 -1
        -1 -1 1
        -1 1 -1
        1 -1 -1
        -1 -1 -1
        0 1/phi, phi
        0 -1/phi, phi
        0 1/phi, -phi
        0, -1/phi, -phi
        1/phi, phi, 0
        -1/phi, phi, 0
        1/phi, -phi, 0
        -1/phi, -phi, 0
        phi, 0, 1/phi
        -phi, 0, 1/phi
        phi, 0, -1/phi
        -phi, 0, -1/phi
        ];
    faces = [9, 10, 5, 18, 2
        9, 10, 3, 17, 1
        10, 5, 16, 15, 3
        3,15,7,19,17
        17,19,4,13,1
        1,13,14,2,9
        2,14,6,20,18
        18,20,8, 16,5
        16,8,12,7,15
        7,12,11, 4, 19
        13,4,11,6,14
        6,11,12,8,20
        ];

   polyhedron.vertices=vert;
   polyhedron.faces=faces;
   if ~isempty(edgelength)
       polyhedron.vertices = polyhedron.vertices/(2/phi)*edgelength;
   end

elseif any(strcmp(ptype,{'Trapezorhombic Dodecahedron', 'TD'}))
    % Trapezorhombic Dodecahedron
   vert = [-1.000000000000000  -0.577350269189626  -0.816496580927726
                   0                   0  -1.224744871391589
  -1.000000000000000   0.577350269189626  -0.408248290463863
                   0   1.154700538379252  -0.816496580927726
                   0  -1.154700538379252  -0.408248290463863
                   0  -1.154700538379252   0.408248290463863
   1.000000000000000  -0.577350269189626  -0.816496580927726
  -1.000000000000000  -0.577350269189626   0.816496580927726
   1.000000000000000   0.577350269189626  -0.408248290463863
  -1.000000000000000   0.577350269189626   0.408248290463863
   1.000000000000000  -0.577350269189626   0.816496580927726
   1.000000000000000   0.577350269189626   0.408248290463863
                   0                   0   1.224744871391589
                   0   1.154700538379252   0.816496580927726];
   faces = [ 1     2     7     5
     1     2     4     3
     1     5     6     8
     1     3    10     8
     2     4     9     7
     3    10    14     4
     4     9    12    14
     5     6    11     7
     6     8    13    11
     7     9    12    11
     8    10    14    13
    11    12    14    13];
   polyhedron.vertices=vert;
   polyhedron.faces=faces;
   if ~isempty(edgelength)
       polyhedron.vertices = polyhedron.vertices/sqrt(3)*edgelength;
   end
                   
elseif any(strcmp(ptype,{'rhombicdodecahedron', 'rhombic dodecahedron', 'RD'}))
    phi = (1+sqrt(5))/2;

vert = [1,1,1;
    -1,1,1;
    1,-1,1;
    1,1,-1;
    -1,-1,1;
    -1,1,-1;
    1,-1,-1;
    -1,-1,-1;
    0, 0, 2;
    0, 0, -2;
    2, 0, 0;
    -2, 0, 0;
    0, 2, 0;
    0 -2, 0];

faces = [1 13 4 11;
    1 9 2 13;
    1 9 3 11;
    4 10 6 13;
    4 10 7 11;
    2 12 6 13;
    2 12 5 9;
    6 12 8 10;
    12 8 14 5;
    3 11 7 14;
    7 14 8 10;
    3 14 5 9];

   polyhedron.vertices=vert;
   polyhedron.faces=faces;
   if ~isempty(edgelength)
       polyhedron.vertices = polyhedron.vertices/sqrt(3)*edgelength;
   end

elseif any(strcmp(ptype,{'longRD'}))
    if isempty(edgelength)
        edgelength = 1;
    end
    if isempty(tetheight)
        tetheight = edgelength;
    end
    HofRD = 1/sqrt(3)*edgelength;
    tetheight = tetheight-2*HofRD;
vert = [1,1,1;
    -1,1,1;
    1,-1,1;
    1,1,-1;
    -1,-1,1;
    -1,1,-1;
    1,-1,-1;
    -1,-1,-1;
    0, 0, 2;
    0, 0, -2;
    2, 0, 0;
    -2, 0, 0;
    0, 2, 0;
    0 -2, 0];
    vert = vert/sqrt(3)*edgelength;
    t = vert(:,3) == 0;
    vert0 = vert(t, :);
    t = vert(:,3)<=0;
    vert(t, 3) = vert(t, 3) - tetheight/2;
    t = vert(:,3)>0;
    vert(t, 3) = vert(t, 3) + tetheight/2;
    vert0(:,3) = vert0(:,3) + tetheight/2;
    vert = [vert; vert0];

   polyhedron.vertices=vert;
   polyhedron.faces = findfaces(vert);

   
elseif (strcmp(ptype,'icosahedron'))
    % ? = (1 + ?5) / 2 : golden ratio
    phi = (1+sqrt(5))/2;
   polyhedron.vertices=[...
    0, 1, phi;
    0, -1, phi;
    0, 1, -phi;
    0, -1, -phi;
    1, phi, 0;
    -1, phi, 0;
    1, -phi, 0;
    -1, -phi, 0;
    phi, 0, 1;
    -phi, 0, 1;
    phi, 0, -1;
    -phi, 0, -1;
    ];
faces = [1,2,10;
    1,10,6;
    1,6,5;
    1,5,9;
    1,9,2;
    4,3,11;
    4,11,7;
    4,7,8;
    4,8,12
    4,12,3;
    12,10,8;
    10,8,2;
    8,2,7;
    2,7,9;
    7,9,11;
    9,11,5;
    11,5,3;
    5,3,6;
    3,6,12;
    6,12,10];

   if ~isempty(edgelength)
       polyhedron.vertices = polyhedron.vertices/2*edgelength;
   end

   polyhedron.faces=faces;

elseif (strcmp(ptype,'pentagonalbipyramid'))
    % ? = (1 + ?5) / 2 : golden ratio
    if isempty(edgelength)
        edgelength = 1;
    end
    if isempty(tetheight)
        tetheight = edgelength;
    end
    R = edgelength/sqrt((5-sqrt(5))/2);
    v0 = [0, R, 0];
    Nbottom = 5;
    v = v0;
    for i=1:Nbottom-1
        M = [cos(2*pi/Nbottom*i),sin(2*pi/Nbottom*i),0;-sin(2*pi/Nbottom*i),cos(2*pi/Nbottom*i),0;0,0,1];
        vn = v0*M;
        v = [v;vn];
    end
    v = [v;0,0,tetheight/2;0,0,-tetheight/2];
   polyhedron.vertices=v;
   polyhedron.faces = findfaces(v);

    
elseif (strcmp(ptype,'truncatedcube'))
    % ? = (1 + ?5) / 2 : golden ratio
    xi = sqrt(2)-1;
    
%    if numel(varargin)==3
%        xi =varargin{2};
%        edgelength = varargin{3};
%    end
    if isempty(edgelength)
        edgelength = 1;
    end
    if (tetheight == sqrt(2)*edgelength/2)
        polyhedron = polyhedra('cuboctahedron', edgelength/2*sqrt(2));
        return
    end
    if isempty(tetheight)
        L = edgelength;
       v=[...
           xi, 1, 1
           1, xi, 1
           1, 1, xi];

       v = [v;
           [-v(:,1), v(:,2), v(:,3)];
           [v(:,1), -v(:,2), v(:,3)];
           [v(:,1), v(:,2), -v(:,3)];
           [-v(:,1), -v(:,2), v(:,3)];
           [-v(:,1), v(:,2), -v(:,3)];
           [v(:,1), -v(:,2), -v(:,3)];
           [-v(:,1), -v(:,2), -v(:,3)];];
       v = reducehkl(v);
       polyhedron.vertices = v/xi*L/2;
    else
        L = edgelength; % edge length of octahedron.
        b = tetheight; % edge length of truncated pyramid.
        if (b>(sqrt(2)*L/2))
            error('truncated egde should be at less than sqrt(2)*L/2')
        end
        t1=[...
           L/2-b/sqrt(2), L/2, L/2
           L/2, L/2-b/sqrt(2), L/2
           L/2, L/2, L/2-b/sqrt(2)];
        % truncation for [1,0,0]
        t=[];
        for n=1:3
            R = rotate_around_vector([0,0,1], 90*n);
            t = [t;t1*R'];
        end
        t = [t1;t];
        R = rotate_around_vector([1,0,0], 180);
        t = [t;t*R'];
        v = t;
        polyhedron.vertices = t;
    end
   try
   polyhedron.faces = findfaces(v);
   catch
   end
   
%   if ~isempty(edgelength)
%       polyhedron.vertices = polyhedron.vertices/2*edgelength;
%   end
elseif (strcmp(ptype, 'concavecube'))
    h = tetheight/edgelength;
	v = [0 0 0; 1 0 0; 1 1 0; 0 1 0; ...
         0 0 1; 1 0 1; 1 1 1; 0 1 1;] ;    
     v = [v; 1/2, 1/2, 1-h; 1/2, 1/2, h;...
         1-h, 1/2, 1/2; h, 1/2, 1/2;...
         1/2, 1-h, 1/2; 1/2, h, 1/2];
     face = [1, 2, 14;
         1, 5, 14;
         5, 6, 14;
         6, 2, 14;
         2, 3, 11;
         3, 7, 11;
         7, 6, 11;
         6, 2, 11;
         3, 4, 13;
         4, 8, 13;
         8, 7, 13;
         7, 3, 13;
         4, 1, 12;
         4, 8, 12;
         8, 5, 12;
         5, 1, 12;
         5, 6, 9;
         6, 7, 9;
         7, 8, 9;
         8, 5, 9;
         1, 2, 10;
         2, 3, 10;
         3, 4, 10;
         4, 1, 10;];
         
    v = (v-0.5)*edgelength;
    polyhedron.vertices = v;
    polyhedron.faces = face;
elseif (strcmp(ptype, 'convexcube'))
    h = tetheight/edgelength;
	v = [0 0 0; 1 0 0; 1 1 0; 0 1 0; ...
         0 0 1; 1 0 1; 1 1 1; 0 1 1;] ;    
     v = [v; 1/2, 1/2, 1+h; 1/2, 1/2, -h;...
         1+h, 1/2, 1/2; -h, 1/2, 1/2;...
         1/2, 1+h, 1/2; 1/2, -h, 1/2];
     face = [1, 2, 14;
         1, 5, 14;
         5, 6, 14;
         6, 2, 14;
         2, 3, 11;
         3, 7, 11;
         7, 6, 11;
         6, 2, 11;
         3, 4, 13;
         4, 8, 13;
         8, 7, 13;
         7, 3, 13;
         4, 1, 12;
         4, 8, 12;
         8, 5, 12;
         5, 1, 12;
         5, 6, 9;
         6, 7, 9;
         7, 8, 9;
         8, 5, 9;
         1, 2, 10;
         2, 3, 10;
         3, 4, 10;
         4, 1, 10;];
         
    v = (v-0.5)*edgelength;
    polyhedron.vertices = v;
    polyhedron.faces = face;
         
elseif (strcmp(ptype,'truncatedoctahedron'))
    if isempty(edgelength)
        edgelength = 1;
    end
    if isempty(tetheight)
    % ? = (1 + ?5) / 2 : golden ratio
       v=[0, 1, 2];
       v = [v;
           [-v(:,1), v(:,2), v(:,3)];
           [v(:,1), -v(:,2), v(:,3)];
           [v(:,1), v(:,2), -v(:,3)];
           [-v(:,1), -v(:,2), v(:,3)];
           [-v(:,1), v(:,2), -v(:,3)];
           [v(:,1), -v(:,2), -v(:,3)];
           [-v(:,1), -v(:,2), -v(:,3)];];
       v = [v;
           [v(:,1), v(:,3), v(:,2)];
           [v(:,2), v(:,1), v(:,3)];
           [v(:,2), v(:,3), v(:,1)];
           [v(:,3), v(:,2), v(:,1)];
           [v(:,3), v(:,1), v(:,2)];];
       v = reducehkl(v);
       polyhedron.faces = findfaces(v);
       polyhedron.vertices = v;
       if ~isempty(edgelength)
           polyhedron.vertices = polyhedron.vertices/sqrt(2)*edgelength;
       end
    else
        % draw octahedron whose each vertices pointing xyz axes.
        % then, its edge length is sqrt(2)
        %v = [1, 0, 0;
        %    -1, 0, 0;
        %    0, 1, 0;
        %    0, -1, 0;
        %    0, 0, 1;
        %    0, 0, -1;]
        L = edgelength; % edge length of octahedron.
        b = tetheight; % edge length of truncated pyramid.
        % truncation for [1,0,0]
%         t1 = [(L-b)/sqrt(2), b/sqrt(2)/2, 0;
%             (L-b)/sqrt(2), -b/sqrt(2)/2, 0;
%             (L-b)/sqrt(2), 0, b/sqrt(2)/2;
%             (L-b)/sqrt(2), 0, -b/sqrt(2)/2];
        t1 = [b/2, -b/2, (L-b)/sqrt(2);
            L/2-b, -L/2, 0;
            L/2, -L/2+b, 0;
            L/2-b/2, -(L/2-b/2), b/sqrt(2)];
        t=[];
        for n=1:4
            R = rotate_around_vector([0,0,1], 90*n);
            t = [t;t1*R'];
        end
        
%        for n=1:2:3
%            R = rotate_around_vector([0,1,0], 90*n);
%            t = [t;t1*R'];
%        end
%        v = [t1;t];
        v =[t;[t(:,1:2), -t(:,3)]];
        v = unique_m(v);
       polyhedron.faces = findfaces(v);
       polyhedron.vertices = v;
    end
    
%    ang=pi/4; 
%    R = [cos(ang), -sin(ang), 0; 
%        sin(ang), cos(ang), 0; 
%        0,0,1];
%     v = v*R';
%   polyhedron.vertices = v;
    
elseif sum(strcmp(ptype, {'cuboctahedron', 'cubooctahedron'}))
    % ? = (1 + ?5) / 2 : golden ratio
   v=[1 1 0];
   v = [v;
       [-v(:,1), v(:,2), v(:,3)];
       [v(:,1), -v(:,2), v(:,3)];
       [v(:,1), v(:,2), -v(:,3)];
       [-v(:,1), -v(:,2), v(:,3)];
       [-v(:,1), v(:,2), -v(:,3)];
       [v(:,1), -v(:,2), -v(:,3)];
       [-v(:,1), -v(:,2), -v(:,3)];];
   v = [v;
       [v(:,1), v(:,3), v(:,2)];
       [v(:,2), v(:,1), v(:,3)];
       [v(:,2), v(:,3), v(:,1)];
       [v(:,3), v(:,2), v(:,1)];
       [v(:,3), v(:,1), v(:,2)];];
   v = reducehkl(v);
   polyhedron.faces = findfaces(v);
   %ang=pi/4; 
   %R = [cos(ang), -sin(ang), 0; 
   %    sin(ang), cos(ang), 0; 
   %    0,0,1];
   %v = v*R';
   polyhedron.vertices = v;

   if ~isempty(edgelength)
       polyhedron.vertices = polyhedron.vertices/sqrt(2)*edgelength;
   end

elseif (strcmp(ptype,'truncatedicosahedron'))
    phi = (1+sqrt(5))/2;
    vert = [0, 1, 3*phi; 
    0, -1, 3*phi
    0, 1, -3*phi
    0, -1, -3*phi
    2, (1+2*phi), phi
    -2, (1+2*phi), phi
    2, -(1+2*phi), phi
    -2, -(1+2*phi), phi
    2, (1+2*phi), -phi
    -2, (1+2*phi), -phi
    2, -(1+2*phi), -phi
    -2, -(1+2*phi), -phi
    1, (2+phi), 2*phi
    -1, (2+phi), 2*phi
    1, -(2+phi), 2*phi
    -1, -(2+phi), 2*phi
    1, (2+phi), -2*phi
    -1, (2+phi), -2*phi
    1, -(2+phi), -2*phi
    -1, -(2+phi), -2*phi
];
    vert=[vert;[vert(:,2),vert(:,3), vert(:,1)];[vert(:,3), vert(:,1), vert(:,2)]];
    F = findfaces(vert);
    polyhedron.vertices = vert;
    polyhedron.faces = F;

   if ~isempty(edgelength)
       polyhedron.vertices = polyhedron.vertices/2*edgelength;
   end

else
   error('type must be:tetrahedron, cube, octahedron,dodecahedron,icosahedron');
   
end

function trans = localToGlobal3d(varargin)
%LOCALTOGLOBAL3D Transformation matrix from local to global coordinate system
%
%   TRANS = localToGlobal3d(CENTER, THETA, PHI, PSI)
%   Compute the transformation matrix from a local (or modelling)
%   coordinate system to the global (or world) coordinate system.
%   This is a low-level function, used by several drawing functions.
%
%   The transform is defined by:
%   - CENTER: the position of the local origin into the World coordinate
%       system
%   - THETA: colatitude, defined as the angle with the Oz axis (between 0
%       and 180 degrees), positive in the direction of the of Oy axis.
%   - PHI: azimut, defined as the angle of the normal with the Ox axis,
%       between 0 and 360 degrees
%   - PSI: intrinsic rotation, corresponding to the rotation of the object
%       around the direction vector, between 0 and 360 degrees
%
%   The resulting transform is obtained by applying (in that order):
%   - Rotation by PSI   around he Z-axis
%   - Rotation by THETA around the Y-axis
%   - Rotation by PHI   around the Z-axis
%   - Translation by vector CENTER
%   This corresponds to Euler ZYZ rotation, using angles PHI, THETA and
%   PSI.
%
%   The 'createEulerAnglesRotation' function may better suit your needs as
%   it is more 'natural'.
%
%   Example
%   localToGlobal3d
%
%   See also
%   transforms3d, createEulerAnglesRotation
%
% ------
% Author: David Legland
% e-mail: david.legland@grignon.inra.fr
% Created: 2009-06-19,    using Matlab 7.7.0.471 (R2008b)
% Copyright 2009 INRA - Cepia Software Platform.

%   HISTORY
%   19/08/2009 fix bug in parsing center with 4 args
%   2011-06-21 use degrees

% extract the components of the transform
if nargin == 1
    % all components are bundled in  the first argument
    var     = varargin{1};
    center  = var(1:3);
    theta   = var(4);
    phi     = var(5);
    psi     = 0;
    if length(var) > 5
        psi = var(6);
    end
    
elseif nargin == 4
    % arguments = center, then the 3 angles
    center  = varargin{1};
    theta   = varargin{2};
    phi     = varargin{3};
    psi     = varargin{4};    
    
elseif nargin > 4
    % center is given in 3 arguments, then 3 angles
    center  = [varargin{1} varargin{2} varargin{3}];
    theta   = varargin{4};
    phi     = varargin{5};
    psi     = 0;
    if nargin > 5
        psi = varargin{6};    
    end
end
    
% conversion from degrees to radians
k = pi / 180;

% rotation around normal vector axis
rot1    = createRotationOz(psi * k);

% colatitude
rot2    = createRotationOy(theta * k);

% longitude
rot3    = createRotationOz(phi * k);

% shift center
tr      = createTranslation3d(center);

% create final transform by concatenating transforms
trans   = tr * rot3 * rot2 * rot1;